//package teikade.debug;

import sun.tools.debug.*;
import java.util.*;
import java.io.*;
import java.net.*;
import teikade.util.CodeConverter;

public class Debugger extends Observable implements DebuggerCallback {
    RemoteDebugger	remote;
    RemoteThread	currentThread;
    RemoteThreadGroup	currentThreadGroup;
    PrintStream		out;
    PrintStream		console;
    ObjectInspector	objInspector;
    StackFrameInspector	stackInspector;
    RemoteStackFrame	list[];
    int			selection;

    static final Integer LIST = new Integer(1);
    static final Integer SELECTION = new Integer(2);

    /**
     * The main method for stand alone using
     */
     public static void main(String args[]) {
	Debugger debugger = new Debugger(System.out, System.out);
	DebuggerView.openOn(debugger);
	debugger.debug(args);
     }

/*
     public static void main0(String argv[]) {
	// Get host attribute, if any.
	String localhost;
	try {
	    localhost = InetAddress.getLocalHost().getHostName();
	} catch (Exception ex) {
	    localhost = null;
	}	
	if (localhost == null) {
	    localhost = "localhost";
	}
	String host = null;
	String password = null;
	String classArgs = "";
	String javaArgs = "";
        boolean verbose = false;
	
	for (int i = 0; i < argv.length; i++) {
	    String token = argv[i];
	    if (token.equals("-dbgtrace")) {
		verbose = true;
	    } else if (token.equals("-cs") || token.equals("-checksource") ||
		       token.equals("-noasyncgc") || token.equals("-prof") ||
		       token.equals("-verbose") || token.equals("-v") ||
		       token.equals("-verify") || token.equals("-noverify") ||
		       token.equals("-verifyremote") ||
		       token.equals("-verbosegc") ||
		       token.startsWith("-ms") || token.startsWith("-mx") ||
		       token.startsWith("-ss") || token.startsWith("-oss") ||
		       token.startsWith("-D")) {
		javaArgs += token + " ";
	    } else if (token.equals("-classpath")) {
		if (i == (argv.length - 1)) {
		    System.out.println("No classpath specified.");
		    System.exit(1);
		}
		javaArgs += token + " " + argv[++i] + " ";
	    } else if (token.equals("-host")) {
		if (i == (argv.length - 1)) {
		    System.out.println("No host specified.");
		    System.exit(1);
		}
		host = argv[++i];
	    } else if (token.equals("-password")) {
		if (i == (argv.length - 1)) {
		    System.out.println("No password specified.");
		    System.exit(1);
		}
		password = argv[++i];
	    } else {
		classArgs += token + " ";
	    }
	}
	if (host != null && password == null) {
	    System.out.println("A debug password must be specified for " +
			       "remote debugging.");
	    System.exit(1);
	}
	if (host == null) {
	    host = localhost;
	}

	try {
	    if (!host.equals(localhost) && password.length() == 0) {
		System.out.println(
		    "No password supplied for accessing remote " +
		    "Java interpreter.");
		System.out.println(
		    "The password is reported by the remote interpreter" +
		    "when it is started.");
                System.exit(1);
            }
            new Debugger(host, password, javaArgs, classArgs, 
                    System.out, System.out, verbose);
	} catch(SocketException se) {
	    System.out.println("Failed accessing debugging session on " +
			       host + ": invalid password.");
	} catch(NumberFormatException ne) {
	    System.out.println("Failed accessing debugging session on " +
			       host + ": invalid password.");
	} catch(Exception e) {		
	    System.out.print("Internal exception:  ");
	    System.out.flush();
	    e.printStackTrace();
	}
    }
 */

    Debugger(PrintStream out, PrintStream console) {
	this.out = out;
	this.console = console;
	objInspector = new ObjectInspector();
	stackInspector = new StackFrameInspector();
    }

    Debugger() {
	this(null, null);
    }

    synchronized void debug(String args[]) {
	if (args == null || args.length < 1) {
	    System.out.println("Class name to be debugged is needed");
	    return;
	}
	try {
	    String host = InetAddress.getLocalHost().getHostName();
	    StringBuffer command = new StringBuffer();
	    command.append(System.getProperty("java.home"));
	    command.append("/bin/java_g -debug sun.tools.debug.EmptyApp");
	    Process p = Runtime.getRuntime().exec(command.toString());
	    DataInputStream dis = new DataInputStream(p.getInputStream());
	    String passwordLine = CodeConverter.readLine(dis);
	    int index = passwordLine.indexOf('=');
	    String password = passwordLine.substring(index + 1);
	    remote = new RemoteDebugger(host, password, this, false);
	    String className = args[0];
	    RemoteClass remoteClass = remote.findClass(className);
	    if (remoteClass != null) {
		RemoteField breakMethod = remoteClass.getMethod(args.length == 1 ? "main" : args[1]);
		remoteClass.setBreakpointMethod(breakMethod);
		remote.run(args.length, args);
	    } else {
		System.out.println("Class " + className + " not found.");
		System.exit(0);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    public synchronized void printToConsole(String msg) throws Exception {
	if (console != null) {
            console.print(msg);
            console.flush();
	}
    }

    public void println(String msg) throws Exception {
	if (out != null) {
            out.println(msg);
	}
    }

    public void breakpointEvent(RemoteThread t) throws Exception {
        setCurrentThread(t);
    }

    public void exceptionEvent(RemoteThread t, String errorText) 
      throws Exception {
	setCurrentThread(t);
    }

    public void threadDeathEvent(RemoteThread t) throws Exception {
        if (t == currentThread) {
            setCurrentThread(null);
        }
    }

    public void quitEvent() throws Exception {
	// we should do something ...
    }

    void changed(Integer what) {
	setChanged();
	notifyObservers(what);
    }

    private void setCurrentThread(RemoteThread t) throws Exception {
	if (t != null) {
	    currentThread = t;
	    // currentThreadGroup = t.getThreadGroup();
	    list = t.dumpStack();
	    selection = 0;
	    changed(LIST);
	    stackInspector.setList(t.getStackVariables());
	    RemoteStackVariable thisRsv = list[0].getLocalVariable("this");
	    if (thisRsv != null) {
	        // instance method
	        objInspector.setList((RemoteObject)thisRsv.getValue());
	    } else {
	        // static method
	        objInspector.setList(list[0].getRemoteClass());
	    }
	} else {
	    list = null;
	    selection = -1;
	    changed(LIST);
	}
    }

    void selected(int index) {
	if (selection == index) {
	    selection = -1;
	} else {
	    selection = index;
	}
	changed(SELECTION);
    }

    String getSourceCode(int lines) {
	try {
	    if (selection != -1) {
		DataInputStream dis =
		    new DataInputStream(
		    new BufferedInputStream(
			list[selection].getRemoteClass().getSourceFile()));
		int start = list[selection].getLineNumber();
		if (start > lines / 2) {
		    start = start - lines / 2;
		} else {
		    start = 0;
		}
		for (int i = 0; i < start ; i++) {
		    String dummy = CodeConverter.readLine(dis);
		}
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < lines; i++) {
		    String line =  CodeConverter.readLine(dis);
		    if (line == null) break;
		    buf.append(line);
		}
		dis.close();
		return buf.toString();
	    } else {
		return "" + selection;
	    }
	} catch (Exception e) {
	    return "(could not get code)";
	}
    }
}
