/*
 * Copyright (c) 1995 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.gui;

import teikade.sys.*;
import teikade.util.*;
import teikade.gui.dialog.TeikadeFileDialog;
import teikade.gui.menu.*;
import java.util.Enumeration;
import java.awt.*;
import teikade.lang.*;

public class ChangeListFrame extends ListingFrame {
    private String changefile = null; 
    private CodePath filepath;

    public static void openOn(CodePath cp) {
	ChangeListFrame frame = new ChangeListFrame(cp);
	ChangeList changes = cp.projectManager().changes();
	int count = changes.count();
	if (count == 0) count = 100;
	String req = (new Integer(count)).toString();
	req = SystemManager.request("How many changes to be listed? ", req);
	if (req == null || req.trim().length() == 0) return;
	try {
	    count = Integer.parseInt(req);
	} catch (Exception e) {
	    count = 100;
	}
	try {
	    frame.start((new ChangeList(changes.file())).elementsFromFile(count, true));
	} catch (Exception e) {
	    SystemManager.information(changes.file().getPath() + ": " + e.getMessage());
	    return;
	}
	frame.keepValue( new CodePath(cp), null );
	frame.open();
    }

    protected boolean select(int newSelection) throws TeikadeException {
	boolean createClass = true;
	boolean makeClass = false;
	CodePath cp;
	ClassManager cm = null;
	if (newSelection < 0 || newSelection == selection) {
	    cp = CodePath.nullCodePath;
	    newSelection = -1;
	} else {
	    ListItem item = itemAt(newSelection);
	    ChangeList chgCode;
	    if( changefile == null ) {
		chgCode = filepath.projectManager().changes();
	    } else {
		chgCode = new ChangeList(TeikadeFile.create(changefile));
	    }
	    String newCode = chgCode.getCode(((Changes)item));

	    Object ob2 = ((Changes)item).getcodePath();
	    if(!(ob2 instanceof CodePath)) {
		String names[] = CodePath.pathToNames((String)ob2);
		CodePath cp3 = new CodePath(names);
		if (cp3.packageManager() == null) {
		    PackageManager pkgMan =
		    PackageManager.findPackage(names[CodePath.PACKAGE]);
		    if (pkgMan == null) {	// no package found. should create one?
			String msg = "Which project should package " +
				    names[CodePath.PACKAGE] +
				 " go into?";
			String projName = names[CodePath.PROJECT];
			projName = SystemManager.request(msg, projName);
			if( projName == null ) {
			    cp3 = CodePath.nullCodePath;
			    createClass = false;
			} else {
			    ProjectManager projMan = SystemManager.findProject(projName);
			    if (projMan == null) {
				SystemManager.information(
				    "Project " + projName + " does not exist. " +
				    "Create it before refering '" + ob2 + "'.");
				cp3 = CodePath.nullCodePath;
			        createClass = false;
			    } else {
				cp3.projectManager(projMan);
				pkgMan = projMan.createPackageAfter(names[CodePath.PACKAGE], null);
			    }
			}
		    }
		    if( cp3 != CodePath.nullCodePath ) {
			cp3.packageManager(pkgMan);
		    }
		}

		if( cp3 != CodePath.nullCodePath ) {
		    if( names[CodePath.CLASS] != null && cp3.classManager() == null ) {
			if( names[CodePath.METHOD] != null ) {
			    SystemManager.information("Method " + names[CodePath.METHOD] + " does not create. " + 
				"Because Class " + names[CodePath.CLASS] + " does not exist. ");
			    cp3 = CodePath.nullCodePath;
			    createClass = false;
			} else {
			    String msg = "Class " + names[CodePath.CLASS] + " does not exist. " +
				"Create it?";
			    if( SystemManager.confirm(msg) ) {
				String fname = names[CodePath.CLASS] + ".java";
				String description = "";
				String newCodeSpec = newCode.substring(0, newCode.indexOf("{") - 1);
				ClassSpec newSpec = null;
				CodeReader reader = new CodeReader(newCode);
				int nest = reader.nextDescription();
				description = reader.description();
				if (nest > 0) reader.nextDescription(false);
				if (reader.nextDescription() >= 0) {
				    SystemManager.information(names[CodePath.CLASS] + ": too many description or not enough open brace");
				    cp3 = CodePath.nullCodePath;
				    createClass = false;
				}
				if( cp3 != CodePath.nullCodePath ) {
				    try {
					newSpec = new ClassSpec(description);
				    } catch (DescriptionException ce) {
					cp3 = CodePath.nullCodePath;
					createClass = false;
					throw new DescriptionException(ce.getMessage());
				    }
				}
				if( cp3 != CodePath.nullCodePath ) {
				    if( newSpec == null ) {
					cp3 = CodePath.nullCodePath;
					createClass = false;
				    } else {
					if (!newSpec.isPublic()) {
					    fname = SystemManager.requestFileName(
					        "What is the filename for the class " + newSpec.className() + "?",
					        cp3.packageManager().absolutePath(),fname);
					    if (fname == null) {
						cp3 = CodePath.nullCodePath;
						createClass = false;
					    }
					}
				    }
				}
				if( cp3 != CodePath.nullCodePath ) {
				    SourceFileManager sfm = cp3.packageManager().findSource(fname);
				    if (sfm == null) {
					sfm = cp3.packageManager().createSourceFileManager(fname);
				    }
				    if (!sfm.isEditable()) {
					cp3 = CodePath.nullCodePath;
					createClass = false;
				    } else {
					cm = new ClassManager(sfm, newSpec);
					if (cm.resolveDependancy()) {
					    sfm.addChild(cm);
					    cp3.packageManager().updateSourcesFile();
					    cm.created(newCode);
					    try {
						Enumeration m = cm.methods();
						while (m.hasMoreElements()) {
						    MethodManager mm = (MethodManager)m.nextElement();
						    mm.created(mm.getCode());
						}
					    } catch (TeikadeException e) {
						SystemManager.exception("Could not read source file: ", e);
					    }
					
					    makeClass = true;
					} else {
					    cp3 = CodePath.nullCodePath;
					    createClass = false;
					}
				    }
				}	
			    } else {
				SystemManager.information("Aborting reference to: " + ob2);
				cp3 = CodePath.nullCodePath;
				createClass = false;
			    }
			}
		    }
		}
		ob2 = cp3;
	    }
	    cp = (CodePath)ob2;

	    if (selection != -1 && item.isSameCode(itemAt(selection))) {
		// same code, no need to set code, move cursor only
	        selectCommonAction1( newSelection, item );
		return true;
	    }
	}
	if (acceptWithConfirm() && createClass == true) {
	    selectCommonAction2( newSelection, cp );
	    if( makeClass == true ) {
		if( cm == null ) {
		    return false;
		} else {
		    current.classManager(cm);
    		    TeikadeFrame.updateNotify(current);
		    update();
		    current.classManager().sync();
		    updateButtonActivity(false, false);
		}
	    }
	    return true;
	} else {
	    selectCommonAction3();
	    return false;
	}
    }

void keepValue(CodePath path, String name) {
    filepath = path;
    changefile = name;
}

    public static void openOn(String fileName) {
	ChangeListFrame frame = new ChangeListFrame(fileName);
	ChangeList changes = new ChangeList(TeikadeFile.create(fileName));
	try {
	    frame.start(changes.elementsFromFile(Integer.MAX_VALUE, false));
	} catch (Exception e) {
	    SystemManager.information(fileName + ": " + e.getMessage());
	    return;
	}
	frame.keepValue( null, fileName );
	frame.open();
    }

    public ChangeListFrame(CodePath cp) {
	super(cp, "Changes on: " + cp.projectName());
    }

    public ChangeListFrame(String fileName) {
	super(CodePath.nullCodePath, "Changes from: " + fileName);
    }

    protected TeikadeMenuBar createMenuBar() {
	return new ChangeListMenuBar();
    }

    public boolean action(Event ev, Object arg) {
	if (ev.target != list) {
	    return super.action(ev, arg);
	}
	selection = list.getSelectedIndex();
        if (selection >= 0) {
	    current = itemAt(selection).codePath();
        } else {
	    current = CodePath.nullCodePath; 
	}
	updateCurrent();
	return true;
    }

    public void updateCode() {
        if (selection == -1) {
            setCode(getMessages());
	} else {
	    setCode(((Changes)itemAt(selection)).getCode());
	}
    }

    /**
     * accept code
     */
    public boolean accept() {
	return doAccept();
    }

    public void loadFromFile(boolean merge) {
	TeikadeFile f = startup.projectManager().changes().file();
	String fname = TeikadeFileDialog.request(this, "Filename to load", f.getParent(), f.getName());
	if (fname == null) return;
	try {
	    Enumeration en = (new ChangeList(fname)).elementsFromFile(Integer.MAX_VALUE, true);
	    if (merge) {
		start(en);
	    } else {
		restart(en);
	    }
	} catch (Exception e) {
	    SystemManager.information(fname + ": " + e.getMessage());
	}
    }

    public void saveIntoFile(boolean marked) {
	String fname = TeikadeFileDialog.request(this, "Filename to save");
	if (fname == null) return;
	ChangeList cl = new ChangeList(fname);
	Enumeration en = itemElements();
	while (en.hasMoreElements()) {
	    Changes chg = (Changes)en.nextElement();
	    if (!marked || chg.isMarked()) {
		try {
		    cl.addChanges(chg.codePath(), chg.action(), chg.getCode());
		} catch (Exception e) {
		    SystemManager.information(fname + ": " +
					  chg.codePath().printString() + ": " +
					  e.getMessage());
		}
	    }
	}
    }

    public ListingFrame copyCreate(CodePath cp) {
	return new ChangeListFrame(cp);
    }
}
