/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.Event;
import java.util.Vector;

class EventDelegater implements Runnable {
    private EventHandler handler;
    Thread eventHandleThread;
    private Vector events;

    EventDelegater(EventHandler handler) {
	this.handler = handler;
	events = new Vector();
	eventHandleThread = new Thread(this,this.getClass().getName());
	eventHandleThread.start();
    }

    synchronized void dispose() {
	if (eventHandleThread != null) {
	    eventHandleThread = null;
	    events = null;
	    handler = null;
	    notifyAll();
	}
    }

    boolean delegate(Event event) {
	switch (event.id) {
	case Event.ACTION_EVENT:
	case Event.LIST_SELECT:
	case Event.LIST_DESELECT:
	case Event.WINDOW_DESTROY:
	    if (eventHandleThread != null) {
		putEvent(event);
		return false;
	    }
	}
	if (handler != null) return handler.postEvent0(event);
	return false;
    }

    public void run() {
	Event event;
	while ((event = getEvent()) != null) {
	    try {
		handler.postEvent0(event);
	    } catch (Throwable e) {
		e.printStackTrace();
	    }
	}
	eventHandleThread = null;
    }

    private synchronized final Event getEvent() {
	while (eventHandleThread != null) {
	    if (events.size() > 0) {
		Event event = (Event)events.firstElement();
		events.removeElementAt(0);
		return event;
	    }
	    try {
		wait(1000);
	    } catch (InterruptedException e) {
		e.printStackTrace();
		return null;
	    }
	}
	return null;
    }

    private synchronized final void putEvent(Event event) {
	events.addElement(event);
	notifyAll();
    }
}
