/*
 * Copyright (c) 1996 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.component;

import java.awt.*;
import teikade.sys.SystemManager;

public class TagCard extends Panel {
    private static int INSET = 10;
    private TagViewer tagViewer;
    private CardViewer cardViewer;
    static final int VGAP = 2;
    static final int HGAP = 4;
    static final int SHADOW = 3;

    public TagCard() {
	tagViewer  = new TagViewer();
	cardViewer = new CardViewer();

	CardLayout cardLayout = new CardLayout();
	cardViewer.setLayout(cardLayout);
	tagViewer.setModel(cardLayout, cardViewer);

	setLayout(new BorderLayout());
	super.add("North", tagViewer);
	super.add("Center", cardViewer);
    }

    public Component add(String name, Component component) {
	tagViewer.add(name, component);
	return cardViewer.add(name, component);
    }

    public void select(String name) {
	tagViewer.select(name);
    }

    public void paint(Graphics g) {
	Rectangle rect = cardViewer.bounds();
	g.setColor(getBackground().darker());
	g.fillRect(rect.x - HGAP + SHADOW, rect.y - VGAP + SHADOW,
		   rect.width + HGAP * 2, rect.height + VGAP * 2);
	g.setColor(getBackground());
	g.fill3DRect(rect.x - HGAP, rect.y - VGAP,
		     rect.width + HGAP * 2, rect.height + VGAP * 2,
		     true);
    }

    public Insets insets() {
	if (SystemManager.onWin32 && SystemManager.onJDK1_1) {
	    return new Insets(INSET, INSET, INSET + SHADOW, INSET + SHADOW);
	}
	Insets in = super.insets();
	in.top    += INSET;
	in.left   += INSET;
	in.bottom += INSET + SHADOW;
	in.right  += INSET + SHADOW;
	return in;
    }

/*
    public boolean tabbable() {
	return true;
    }

    public boolean gotFocus(Event evt, Object what) {
	return true;
    }

    public boolean lostFocus(Event evt, Object what) {
	return true;
    }
*/
}

class CardViewer extends Panel {
    private static int INSET = 5;

    public Insets insets() {
	if (SystemManager.onWin32 && SystemManager.onJDK1_1) {
	    return new Insets(INSET, INSET, INSET, INSET);
	}
	Insets in = super.insets();
	in.top    += INSET;
	in.left   += INSET;
	in.bottom += INSET;
	in.right  += INSET;
	return in;
    }
}

class TagViewer extends Panel {
    private CardLayout cardLayout;
    private CardViewer cardViewer;

    public TagViewer() {
	setLayout(new TagLayout());
    }

    public void setModel(CardLayout cardLayout, CardViewer cardViewer) {
	this.cardLayout = cardLayout;
	this.cardViewer = cardViewer;
    }

    public Component add(String name, Component card) {
	return add(new TagItem(name, card));
    }

    public void select(String name) {
	cardLayout.show(cardViewer, name);
	Component component[] = getComponents();
	for (int i = 0; i < component.length; i++) {
	    component[i].repaint();
	}
    }

    public void paint(Graphics g) {
	Dimension area = size();
	g.setColor(getBackground().brighter());
	g.fillRect(0, area.height - TagCard.VGAP, area.width, 1);
    }

    public boolean mouseUp(Event event, int x, int y) {
	Component comp = locate(x, y);
	if (comp != null && (comp instanceof TagItem)) {
	    select(((TagItem)comp).getText());
	    return true;
	}
	return false;
    }

/*
    public boolean tabbable() {
	return true;
    }

    public boolean gotFocus(Event evt, Object what) {
	return true;
    }

    public boolean lostFocus(Event evt, Object what) {
	return true;
    }
*/

    public boolean keyDown(Event event, int key) {
	if (key == '\n') {
	    if (event.target instanceof TagItem) {
		select(((TagItem)event.target).getText());
		return true;
	    }
	}
	/*
	if (key == '\t') {
	    nextFocus();
	    return true;
	}
	*/
	return false;
    }
}

class TagLayout implements LayoutManager {
    private int gap;

    public TagLayout() {
	this(0);
    }

    public TagLayout(int gap) {
	this.gap = gap;
    }

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
	Insets insets = parent.insets();
	int num = parent.countComponents();
	int w = 0;
	int h = 0;
	for (int i = 0 ; i < num ; i++) {
	    Component comp = parent.getComponent(i);
	    Dimension d = comp.preferredSize();
	    w += d.width + gap * 2;
	    if (h < d.height) h = d.height;
	}
	return new Dimension(insets.left + insets.right + w + gap * 2,  
			     insets.top + insets.bottom + h + gap * 2);
    }

    public Dimension minimumLayoutSize(Container parent) {
	Insets insets = parent.insets();
	int num = parent.countComponents();
	int w = 0;
	int h = 0;
	for (int i = 0 ; i < num ; i++) {
	    Component comp = parent.getComponent(i);
	    Dimension d = comp.minimumSize();
	    w += d.width + gap * 2;
	    if (h < d.height) h = d.height;
	}
	return new Dimension(insets.left + insets.right + w + gap * 2,
			     insets.top + insets.bottom + h + gap * 2);
    }

    public void layoutContainer(Container parent) {
	int num = parent.countComponents();
	if (num == 0) return;
	Insets insets = parent.insets();
	int x = insets.left + gap;
	int y = insets.top + gap;
	int h = parent.size().height - gap * 2;
	for (int i = 0; i < num; i++) {
	    Component comp = parent.getComponent(i);
	    int w = comp.preferredSize().width;
	    comp.reshape(x, y, w, h);
	    x += w + gap * 2;
	}
    }
}

class TagItem extends Canvas {
    private String name;
    private Component card;
    private boolean focused = false;
    private static final int INSET = 6;
    private static final int TAG_GAP = TagCard.SHADOW + 2;

    public TagItem(String name, Component card) {
	this.name = name;
	this.card = card;
    }

    public String getText() {
	return name;
    }

    public Dimension preferredSize() {
	Font f = getFont();
	int height = f.getSize();
	int width  = getFontMetrics(f).stringWidth(name);
	return new Dimension(width + INSET * 2, height + INSET * 2);
    }

    public Dimension minimumSize() {
	Font f = getFont();
	int height = f.getSize();
	int width = getFontMetrics(f).stringWidth(name);
	return new Dimension(width, height);
    }

    public void paint(Graphics g) {
	Dimension dim = size();
	int shadow = card.isVisible() ? TagCard.SHADOW : 0;
	FontMetrics metrics = g.getFontMetrics();
	int x = (dim.width - metrics.stringWidth(name)) / 2;
	int y = (dim.height + metrics.getHeight()) / 2 - metrics.getMaxDescent();
	g.drawString(name, x - shadow, y - shadow);
	if (focused) {
	    g.drawRect(INSET - shadow, INSET - shadow, dim.width - INSET * 2, dim.height - INSET * 2);
	}
	g.setColor(getBackground());
	int h = dim.height - TagCard.VGAP;
	int w = dim.width - TAG_GAP;
	if (card.isVisible()) {
	    // draw frame rectangle
	    g.draw3DRect(0, 0, w, h, true);
	    // erase bottom line
	    x = 1;
	    g.fillRect(x, h, w - x, 1);
	    // draw tail of bottom line
	    g.setColor(getBackground().brighter());
	    g.fillRect(w, h, TAG_GAP, 1);
	    // draw shadow
	    x = w + 1;
	    g.setColor(getBackground().darker());
	    g.fillRect(x, TagCard.SHADOW, TagCard.SHADOW, h - TagCard.SHADOW);
	} else {
	    // draw frame rectangle
	    x = TagCard.SHADOW;
	    y = TagCard.SHADOW;
	    g.draw3DRect(x, y, w, h - y, true);
	    // draw bottom line
	    g.setColor(getBackground().brighter());
	    g.fillRect(0, h, dim.width, 1);
	}
    }

    public boolean tabbable() {
	return true;
    }

    public boolean gotFocus(Event evt, Object what) {
	focused = true;
	repaint();
	return true;
    }

    public boolean lostFocus(Event evt, Object what) {
	focused = false;
	repaint();
	return true;
    }

    public boolean keyDown(Event event, int key) {
	if (key == '\t') {
	    nextFocus();
	    return true;
	}
	return false;
    }
}
