/*
 * Copyright (c) 1995-1998 PFU Limited.
 *	author Kazuhisa Shirakami
 */
package teikade.gui.editor;

import java.awt.*;
import java.util.Hashtable;
import teikade.sys.SystemManager;
import teikade.sys.TeikadeProperties;
import teikade.lang.DescriptionException;
import teikade.gui.*;
import teikade.gui.component.*;
import teikade.gui.dialog.PropertiesDialog;

public class EditProperties {
    private static final String USE_SYSTEM_CLIPBOARD = "editor.system.clipboard";
    public static int softTab;		// key input
    public static int hardTab;		// display
    private static Hashtable fileTabs = new Hashtable(); // by project name
    public static int selectedHandler;
    public static boolean useSystemClipboard = true;

    private static EditKeyHandler viewKeyHandler;
    private static EditKeyHandler editKeyHandler;
    private static final String handlers[][] = {
	{ "system default", null },
	{ "simple", "Simple" },
	{ "emacs like", "Emacs" },
	{ null, null }
    };

    static {
	int soft = TeikadeProperties.getInteger("editor.tab.soft");
	int hard = TeikadeProperties.getInteger("editor.tab.hard");
	String handler = TeikadeProperties.getString("editor.key");
	softTab = soft > 0 ? soft : 4;
	hardTab = hard > 0 ? hard : 8;
	int select;
/*	if (SystemManager.onJDK1_1 && SystemManager.onWin32) {
 *	    select = 0;
 *	} else 
 */
	if (handler == null) {
	    select = 2;
	} else if (handler.length() == 0) {
	    select = 0;
	} else {
	    select = handlers.length - 2;
	    while (select > 0 && !handler.equals(handlers[select][1])) {
		select--;
	    }
	    if (select < 0) {
		select = handlers.length - 1;
		handlers[select][0] = handler;
		handlers[select][1] = handler;
	    }
	}
	selectEditKeyHandler(select);
	if (SystemManager.onWin32 && "1.2beta4".equals(SystemManager.javaVersion)) {
	    useSystemClipboard = "enable".equals(
			TeikadeProperties.getString(USE_SYSTEM_CLIPBOARD));
	} else {
	    useSystemClipboard = !"disable".equals(
			TeikadeProperties.getString(USE_SYSTEM_CLIPBOARD));
	}
    }

    public static int countHandlers() {
/*	if (SystemManager.onJDK1_1 && SystemManager.onWin32) {
 *	    return 1;
 *	}
 */
	int num = handlers.length;
	if (handlers[num - 1][0] == null) num--;
	return num;
    }

    public static String getHandlerName(int index) {
	if (index < 0 || index >= handlers.length) return null;
	return handlers[index][0];
    }

    public static String selectEditKeyHandler(int select) {
	selectedHandler = select;
	String handler = handlers[select][1];
	if (handler != null && handler.length() > 0) {
	    String name = "teikade.gui.editor." + handler + "ViewKeyHandler";
	    String error = null;
	    try {
		viewKeyHandler = (EditKeyHandler)Class.forName(name).newInstance();
		name = "teikade.gui.editor." + handler + "EditKeyHandler";
		editKeyHandler = (EditKeyHandler)Class.forName(name).newInstance();
		return handler;
	    } catch (ClassNotFoundException e) {
		error = "Not found class " + name + " of editor.key property";
	    } catch (IllegalAccessException e) {
		error = "Illegal access class " + name;
	    } catch (InstantiationException e) {
		error = "Fail to instansiation of class " + name;
	    } catch (ClassCastException e) {
		error = "Class " + name + " was not extend EditKeyHandler";
	    }
	    SystemManager.messageln(error);
	}
	viewKeyHandler = null;
	editKeyHandler = null;
	return handler;
    }

    public static EditKeyHandler getKeyHandler(boolean editable) {
	return editable ? editKeyHandler : viewKeyHandler;
    }

    public static int getFileTabWidth(String projectName) {
	Object value = fileTabs.get(projectName);
	if (value != null) {
	    return ((Integer)value).intValue();
	} else {
	    int width = 8;
	    String property = TeikadeProperties.getString(
					"editor.tab.project." + projectName);
	    if (property != null) {
		try {
		    width = Integer.parseInt(property);
		} catch (Exception e) {
		}
	    }
	    fileTabs.put(projectName, new Integer(width));
	    return width;
	}
    }

    public static void setFileTabWidth(String projectName, int width) {
	int oldWidth = getFileTabWidth(projectName);
	if (width != oldWidth) {
	    fileTabs.put(projectName, new Integer(width));
	    TeikadeProperties.put("editor.tab.project." + projectName,
				  "" + width);
	}
    }

    public static void showDialogOn(Frame f) {
	PropertiesDialog dialog = PropertiesDialog.on(f);
	dialog.select(PropertiesDialog.EDIT);
	dialog.show();
	dialog.hide();
    }

    public static void useSystemClipboard(boolean use) {
	if (useSystemClipboard != use) {
	    useSystemClipboard = use;
	    TeikadeProperties.put(USE_SYSTEM_CLIPBOARD, use ? "enable" : "disable");
	}
    }
}
