/*
 * Copyright (c) 1999 PFU Limited.
 *	author Osamu Satoh
 */
package teikade.sys;

import teikade.gui.TeikadeFrame;
import teikade.gui.component.DMenu;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * The managing object for user plugins.
 */
public class PluginManager {
    private static final Hashtable plugins = new Hashtable();
    private static final Vector pluginNames = new Vector();

    private static final String PLUGIN = "plugin.";

    public static void loadPlugins() {
	Enumeration keys = TeikadeProperties.getKeys();
	while (keys.hasMoreElements()) {
	    String key = (String)keys.nextElement();
	    if (key.startsWith(PLUGIN)) {
		loadPlugin(key, TeikadeProperties.getString(key));
	    }
	}
    }

    private static void loadPlugin(String key, String className) {
	String name = key.substring(PLUGIN.length());
	try {
	    SystemManager.message("Loading the " + name + " plugin: ");
	    Class clazz = Class.forName(className);
	    PluginCommand command = (PluginCommand)clazz.newInstance();
	    command.initialize(name);
	    plugins.put(name, command);
	    pluginNames.addElement(name);
	    SystemManager.messageln("done.");
	    return;
	} catch (ClassNotFoundException cnfe) {
	    SystemManager.message("not found");
	} catch (InstantiationException ie) {
	    SystemManager.message("fail to instantiate");
	} catch (IllegalAccessException iae) {
	    SystemManager.message("can't access to");
	} catch (ClassCastException cce) {
	    SystemManager.message("not implemented PluginComman");
	} catch (PluginException pe) {
	    SystemManager.message("fail to initialize for plugin");
	}
	SystemManager.messageln(" the " + className + " class.");
    }

    public static void addPluginLabels(DMenu menu) {
	Enumeration labels = pluginNames.elements();
	while (labels.hasMoreElements()) {
	    String label = (String)labels.nextElement();
	    menu.add(label);
	}
    }

    public static void updateMenuActivity(DMenu menu, TeikadeFrame frame, CodePath cp) {
	int count = menu.countItems();
	for (int i = 0; i < count; i++) {
	    String label = menu.getItem(i).getLabel();
	    if (isInvocableFor(label, frame, cp)) {
		menu.enable(label);
	    } else {
		menu.disable(label);
	    }
	}
    }

    public static void invoke(String name, TeikadeFrame frame, CodePath cp) {
	PluginCommand command = (PluginCommand)plugins.get(name);
	if (command == null) {
	    // should never fall in here...
	    SystemManager.error("No such plugin... Inconsistent configuration: " + name);
	    return;
	}

	try {
	    command.invoke(name, frame, cp);
	} catch (PluginException e) {
	    SystemManager.exception("Plugin exception: " + name, e);
	}
    }

    public static boolean isInvocableFor(String name, TeikadeFrame frame, CodePath cp) {
	PluginCommand command = (PluginCommand)plugins.get(name);
	if (command == null) {
	    // should never fall in here...
	    SystemManager.error("No such plugin... Inconsistent configuration: " + name);
	    return false;
	}

	return command.isInvocableFor(name, frame, cp);
    }
}
