/*
 * Copyright (c) 1996-1998 PFU Limited.
 *	author Osamu Satoh
 *
 * Wed, 03 Nov 1999 10:24:17 +0000
 *	The "attribute.history.lines" property is offered
 *	by Bill Petheram
 */
package teikade.sys;

import java.util.Properties;
import java.util.Observer;
import java.util.Observable;
import java.util.Enumeration;
import java.awt.Font;
import java.awt.Color;
import java.io.*;
import teikade.util.ParsedProperties;

/*
 * system properties for teikade
 */
public class TeikadeProperties extends Observable {
    private static final Properties defprops = new Properties(System.getProperties());
    private static final ParsedProperties props = new ParsedProperties(defprops);
    public static final TeikadeProperties observers = new TeikadeProperties();

    private static final String defaults[][] = {
    // fonts
	{"font.dialog.name",		"Dialog"},
	{"font.label.name",		"TimesRoman"},
	{"font.default.name",		"Courier"},
	{"font.default.face",		new Integer(Font.PLAIN).toString()},
	{"font.default.size",		"14"},

    // external commands
	{"command.java", 		"%java.home%/bin/java" },
	{"command.javac", 		"%java.home%/bin/javac" },
	{"command.debugger", 		"%java.home%/bin/java teikade.debugger.Debugger " },

    // list item visibility
	{"show.class.public",		"true"},
	{"show.class.friendly",		"true"},
	{"show.method.public",		"true"},
	{"show.method.protected",	"true"},
	{"show.method.priv-protected",	"true"},
	{"show.method.friendly",	"true"},
	{"show.method.private",		"true"},

    // display attribute
	{"attribute.available",		"false"},
	{"attribute.list.foreground",	"black"},
	{"attribute.list.background",	"lightgray"},
	{"attribute.list.class",	"Plain"},
	{"attribute.list.interface",	"Italic"},
	{"attribute.list.public",	"black"},
	{"attribute.list.protected",	"black"},
	{"attribute.list.priv-protected","black"},
	{"attribute.list.friendry",	"black"},
	{"attribute.list.private",	"gray"},

	{"attribute.code.keyword",	"Bold"},
	{"attribute.code.class",	"Italic"},

	{"attribute.browsing.lines",	 "9"},
	{"attribute.files.lines",	"10"},
	{"attribute.listing.lines",	 "7"},
	{"attribute.history.lines",	 "10"},

    // code completion
	{"complete.available",		"false"},
	{"complete.default",		"S-SPC"},
	{"complete.classname",		"S-C-c"},
	{"complete.methodname",		"S-C-m"},
	{"complete.variablename",	"S-C-v"},
    };

    static {
	String home = System.getProperty("java.home");
	String up = File.separator + "..";
	while (home.endsWith(up)) {
	    int cut = home.lastIndexOf(File.separator,
				 home.length() - 1 - up.length());
	    if (cut < 0) break;
	    home = home.substring(0, cut);
	}
	setDefault("java.home", home);
	for (int i = 0; i < defaults.length; i++) {
	    setDefault(defaults[i][0], defaults[i][1]);
	}
	updateProperties();
    }

    public Properties getProperties() {
	return props;
    }

    public static void setDefault(String name, String value) {
	defprops.put(name, value);
    }

    public static String getDefault(String name) {
	return (String)defprops.get(name);
    }

    public static void updateProperties() {
	if (SystemManager.onJDK1_1) {
	    setDefault("font.default.name", "Serif");
	    setDefault("font.label.name",   "Serif");
	    setDefault("font.list.name",    "Monospaced");
	    setDefault("font.code.name",    "Monospaced");
	} else if (SystemManager.onSolaris) {
	    setDefault("font.default.name", "Fixed");
	}
    }

    /*
     * should not create an instance
     */
    private TeikadeProperties() {
    }

    public static synchronized void load(InputStream in) throws IOException {
	props.load(in);
    }

    public static void put(String key, String val) {
	props.put(key, val);
	observers.setChanged();
	observers.notifyObservers(key);
    }

    public static void putWithoutNotify(String key, String val) {
	props.put(key, val);
    }

    /**
     * Save properties to an OutputStream. Use the header as
     * a comment at the top of the file.
     */
    public static synchronized void save(OutputStream out, String header) {
	props.save(out, header);
    }

    /**
     * Get String property
     */
    public static String getParsedString(String key, String defaultKey) {
	return props.getParsedString(key, defaultKey);
    }

    /**
     * Get String property
     */
    public static String getParsedString(String key) {
	return props.getParsedString(key);
    }

    /**
     * Get String property
     */
    public static String getString(String key, String defaultKey) {
	return props.getString(key, defaultKey);
    }

    /**
     * Get String property
     */
    public static String getString(String key) {
	return props.getString(key);
    }

    /**
     * Get integer property
     */
    public static int getInteger(String key, String defaultKey) {
	return props.getInteger(key, defaultKey);
    }

    /**
     * Get integer property
     */
    public static int getInteger(String key) {
	return props.getInteger(key);
    }

    /**
     * Get font property and return proper Font.
     */
    public static Font getFont(String key, int face) {
	return props.getFont(key, face);
    }

    /**
     * Get font property and return proper Font.
     */
    public static Font getFont(String key) {
	return props.getFont(key);
    }

    /**
     * Get color property and return proper Color.
     */
    public static Color getColor(String key) {
	return props.getColor(key);
    }

    /**
     * Return color names.
     */
    public static Enumeration getColorNames() {
	return props.getColorNames();
    }

    /**
     * Returns all property keys.
     */
    public static Enumeration getKeys() {
	return props.keys();
    }

    /**
     * Test main routine
     */
    public static void main(String args[]) {
	if (args.length > 0) try {
	    load(new FileInputStream(args[0]));
	} catch (IOException e) {}
	for (int i = 0; i < defaults.length; i++) {
	    System.out.println(defaults[i][0] +
			"(" + defaults[i][1] + "): " +
			getParsedString(defaults[i][0]));
	}
	for (int i = 1; i < args.length; i++) {
	    System.out.println(args[i] +": " + getParsedString(args[i]));
	}
	   
    }
}
