/*
 * Copyright (c) 1995-1998 PFU Limited.
 *      author Kazuhisa Shirakami
 */
package teikade.util;

import teikade.sys.CodePath;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

public abstract class Compile implements Command, CompileResultListener {
    private String projectName;
    private CodePath codePath;
    private boolean isUpdatedTarget;
    private CompileTarget target;
    private CompileListener listener;

    public static boolean canExecute(CodePath cp) {
	return canExecute(cp.projectName());
    }

    public static boolean canExecute(String projectName) {
	return getInstance(projectName) != null;
    }

    public static Compile getInstance(String projectName) {
	Compile compile = CompileFactory.getInstance().create(projectName);
	if (compile != null) {
	    compile.init(projectName);
	}
	return compile;
    }

    public static Compile getInstance(CodePath cp, boolean updated) {
	Compile compile = CompileFactory.getInstance().create(cp.projectName());
	if (compile != null) {
	    compile.init(cp, updated);
	}
	return compile;
    }

    public void init(String projectName) {
	this.codePath = new CodePath(projectName + '/');
	this.projectName = projectName;
    }

    public void init(CodePath codePath, boolean isUpdatedTarget) {
	this.codePath = codePath;
	this.isUpdatedTarget = isUpdatedTarget;
	this.projectName = codePath.projectName();
    }

    public CodePath getCodePath() {
	return codePath;
    }

    public String getProjectName() {
	return projectName;
    }

    public boolean isSystemDefault() {
	return getProjectName() == null;
    }

    public boolean isUserDefault() {
	return CompileFactory.USER_DEFAULT.equals(getProjectName());
    }

    public boolean isUpdatedTarget() {
	return isUpdatedTarget;
    }

    public CompileTarget getCompileTarget() {
	if (target == null) {
	    target = createCompileTarget();
	    target.update();
	}
	return target;
    }

    protected abstract CompileTarget createCompileTarget();

    public void setCompileListener(CompileListener listener) {
	this.listener = listener;
    }

    public CompileListener getCompileListener() {
	return listener;
    }

    public void executing() {
	if (listener != null) {
	    listener.executing(this);
	}
    }

    public void finished(int value) {
	if (listener != null) {
	    listener.finished(this, value);
	}
    }

    public void aborted(Exception e) {
	if (listener != null) {
	    listener.aborted(this, e);
	}
    }

    public void result(CompileResult result) {
	if (listener != null) {
	    listener.result(this, result);
	}
    }

    public abstract CompileProperties getCompileProperties();
}
